import requests

TH_ARTIFACT_URL = 'https://treeherder.mozilla.org/api/project/{}/artifact/'


def get_request_id_from_job_id(repo_name, job_id):
    """Get buildapi's request_id from Treeherder's artifact API."""
    artifact_url = TH_ARTIFACT_URL.format(repo_name)
    query_params = {'job_id': job_id,
                    'name': 'buildapi'}
    artifact_content = requests.get(artifact_url, params=query_params).json()
    return artifact_content[0]["blob"]["request_id"]



import json
import logging
import os

from handlers import config

from mozillapulse.config import PulseConfiguration
from mozillapulse.consumers import GenericConsumer


logging.basicConfig(format='%(asctime)s %(levelname)s:\t %(message)s',
                    datefmt='%m/%d/%Y %I:%M:%S')
LOG = logging.getLogger()


class PulseConsumer(GenericConsumer):
    """
    Creates a consumer object for the given exchange.

    Documentation for the exchanges:
    https://wiki.mozilla.org/Auto-tools/Projects/Pulse/Exchanges
    """

    def __init__(self, exchange, **kwargs):
        super(PulseConsumer, self).__init__(
            PulseConfiguration(**kwargs), exchange, **kwargs)


def run_pulse(exchange, topic, event_handler, dry_run=True):
    """Listen to a pulse exchange in a infinite loop. Call event_handler on every message."""

    label = 'pulse_actions'
    user = os.environ.get('PULSE_USER')
    password = os.environ.get('PULSE_PW')
    pulse_args = {
        'applabel': label,
        'topic': topic,
        'durable': False,
        'user': user,
        'password': password
    }

    # Pulse consumer's callback passes only data and message arguments
    # to the function, we need to pass dry-run
    def handler_with_dry_run(data, message):
        return event_handler(data, message, dry_run)

    pulse = PulseConsumer(exchange, callback=handler_with_dry_run, **pulse_args)
    LOG.info('Listening on %s, with topic %s' % (exchange, topic))

    while True:
        pulse.listen()


def main():
    with open('./pulse_actions/run_time_config.json', 'r') as f:
        options = json.load(f)

    LOG.setLevel(logging.INFO)
    # requests is too noisy
    logging.getLogger("requests").setLevel(logging.WARNING)

    # Finding the right event handler for the given exchange and topic
    topic_base = options['topic'].split('.')[0]
    try:
        handler_function = config.HANDLERS_BY_EXCHANGE[options['exchange']]["topic"][topic_base]
    except KeyError:
        LOG.error("We don't have an event handler for %s with topic %s." % (options['exchange'], options['topic']))
        exit(1)

    run_pulse(
        exchange=options['exchange'],
        topic=options['topic'],
        event_handler=handler_function,
        dry_run=True)

if __name__ == '__main__':
    main()



"""
This module provides functions to deal with exchange/treeherder/v1/job-actions on buildbot.#.#.

Exchange documentation:
 https://wiki.mozilla.org/Auto-tools/Projects/Pulse/Exchanges#Treeherder:_Job_Actions
"""
from mozci.sources.buildapi import make_retrigger_request, make_cancel_request
from pulse_actions.utils.treeherder import get_request_id_from_job_id


def on_buildbot_event(data, message, dry_run):
    """Act upon buildbot events."""
    # Pulse gives us a job_id and a job_guid, we need request_id.
    repo_name = data['project']
    job_id = data['job_id']
    request_id = get_request_id_from_job_id(repo_name, job_id)

    # Retrigger action
    if data['action'] == 'retrigger':
        make_retrigger_request(repo_name, request_id, dry_run)

    # Cancel action
    elif data['action'] == 'cancel':
        make_cancel_request(repo_name, request_id, dry_run)



from setuptools import setup, find_packages

deps = [
    'mozillapulse',
    'mozci>=0.7.0',
    'requests',
]

setup(name='pulse-actions',
      version='0.1.2',
      description='A pulse listener that acts upon messages with mozci.',
      classifiers=['Intended Audience :: Developers',
                   'License :: OSI Approved :: Mozilla Public License 2.0 (MPL 2.0)',
                   'Natural Language :: English',
                   'Operating System :: OS Independent',
                   'Programming Language :: Python',
                   'Topic :: Software Development :: Libraries :: Python Modules',
                   ],
      author='Alice Scarpa',
      author_email='alicescarpa@gmail.com',
      license='MPL 2.0',
      packages=find_packages(),
      include_package_data=True,
      zip_safe=False,
      install_requires=deps,
      url='https://github.com/adusca/pulse_actions',
      entry_points={
          'console_scripts': [
              'run-pulse-actions = pulse_actions.worker:main'
              ],
          })
